@@ -15,6 +15,7 @@
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "Device.h"
 #include "Dialog_Base_Partition.h"
 #include "Partition.h"
 #include "Utils.h"
@@ -24,8 +25,9 @@
 
 namespace GParted
 {
-	
-Dialog_Base_Partition::Dialog_Base_Partition()
+
+Dialog_Base_Partition::Dialog_Base_Partition(const Device& device)
+ : m_device(device)
 {
 	frame_resizer_base = NULL;
 	GRIP = false ;
@@ -198,8 +200,7 @@ void Dialog_Base_Partition::prepare_new_partition()
 				// If partition size is not an integer multiple of MiB or
 				// the start or end sectors are not MiB aligned, and space
 				// is available, then add 1 MiB to partition so requested
-				// size is kept after GParted_Core::snap_to_mebibyte
-				// method rounding.
+				// size is kept after snap_to_mebibyte() method rounding.
 				Sector partition_size = new_partition->sector_end - new_partition->sector_start + 1;
 				Sector sectors_in_mib = MEBIBYTE / new_partition->sector_size;
 				if (    (    ( partition_size % sectors_in_mib                    > 0 )
@@ -220,6 +221,14 @@ void Dialog_Base_Partition::prepare_new_partition()
 			break;
 	}
 
+	new_partition->free_space_before = Sector(spinbutton_before.get_value_as_int()) * (MEBIBYTE / new_partition->sector_size);
+
+	// If the original before value has not changed, then set indicator to keep start sector unchanged.
+	if ( ORIG_BEFORE == spinbutton_before .get_value_as_int() )
+		new_partition->strict_start = TRUE;
+
+	snap_to_alignment(m_device, *new_partition);
+
 	//update partition usage
 	if ( new_partition->sector_usage_known() )
 	{
@@ -240,14 +249,206 @@ void Dialog_Base_Partition::prepare_new_partition()
 			new_partition->set_sector_usage( new_size, new_size - new_partition->sectors_used );
 		}
 	}
-
-	new_partition->free_space_before = Sector(spinbutton_before.get_value_as_int()) * (MEBIBYTE / new_partition->sector_size);
-
-	//if the original before value has not changed, then set indicator to keep start sector unchanged
-	if ( ORIG_BEFORE == spinbutton_before .get_value_as_int() )
-		new_partition->strict_start = TRUE;
 }
 
+
+void Dialog_Base_Partition::snap_to_alignment(const Device& device, Partition& partition)
+{
+	if (partition.alignment == ALIGN_CYLINDER)
+		snap_to_cylinder(device, partition);
+	else if (partition.alignment == ALIGN_MEBIBYTE)
+		snap_to_mebibyte(device, partition);
+}
+
+
+void Dialog_Base_Partition::snap_to_cylinder(const Device& device, Partition& partition)
+{
+	Sector diff = 0;
+
+	// Determine if partition size is less than half a disk cylinder.
+	bool less_than_half_cylinder = false;
+	if (partition.sector_end - partition.sector_start < device.cylsize / 2)
+		less_than_half_cylinder = true;
+
+	if (partition.type         == TYPE_LOGICAL   ||
+	    partition.sector_start == device.sectors   )
+	{
+		// Must account the relative offset between:
+		// (A) the Extended Boot Record sector and the next track of the
+		//     logical partition (usually 63 sectors), and
+		// (B) the Master Boot Record sector and the next track of the first
+		//     primary partition.
+		diff = (partition.sector_start - device.sectors) % device.cylsize;
+	}
+	else if (partition.sector_start == 34)
+	{
+		// (C) the GUID Partition Table (GPT) and the start of the data
+		//     partition at sector 34.
+		diff = (partition.sector_start - 34) % device.cylsize;
+	}
+	else
+	{
+		diff = partition.sector_start % device.cylsize;
+	}
+	if (diff && ! partition.strict_start)
+	{
+		if (diff < device.cylsize / 2 || less_than_half_cylinder)
+			partition.sector_start -= diff;
+		else
+			partition.sector_start += device.cylsize - diff;
+	}
+
+	diff = (partition.sector_end + 1) % device.cylsize;
+	if (diff)
+	{
+		if (diff < device.cylsize / 2 && ! less_than_half_cylinder)
+			partition.sector_end -= diff;
+		else
+			partition.sector_end += device.cylsize - diff;
+	}
+}
+
+
+void Dialog_Base_Partition::snap_to_mebibyte(const Device& device, Partition& partition)
+{
+	Sector diff = 0;
+	if (partition.sector_start < 2 || partition.type == TYPE_LOGICAL)
+	{
+		// Must account the relative offset between:
+		// (A) the Master Boot Record sector and the first primary/extended partition, and
+		// (B) the Extended Boot Record sector and the logical partition.
+
+		// If strict_start is set then do not adjust sector start.
+		// If this partition is not simply queued for a reformat then
+		// add space minimum to force alignment to next mebibyte.
+		if (! partition.strict_start                        &&
+		      partition.free_space_before == 0              &&
+		      partition.status            != STAT_FORMATTED   )
+		{
+			// Unless specifically told otherwise, the Linux kernel considers extended
+			// boot records to be two sectors long, in order to "leave room for LILO".
+			partition.sector_start += 2;
+		}
+	}
+
+	// Calculate difference offset from Mebibyte boundary.
+	diff = Sector(partition.sector_start % (MEBIBYTE / partition.sector_size));
+
+	// Align start sector only if permitted to change start sector.
+	if (diff && (! partition.strict_start                                      ||
+	            (partition.strict_start && (partition.status == STAT_NEW  ||
+	                                        partition.status == STAT_COPY   ))   ))
+	{
+		partition.sector_start += MEBIBYTE / partition.sector_size - diff;
+
+		// If this is an extended partition then check to see if sufficient space is
+		// available for any following logical partition Extended Boot Record.
+		if (partition.type == TYPE_EXTENDED)
+		{
+			// If there is logical partition that starts less than 2 sectors from
+			// the start of this partition, then reserve a mebibyte for the EBR.
+			int index_extended = find_extended_partition(device.partitions);
+			if (index_extended >= 0)
+			{
+				for (unsigned int i = 0; i < device.partitions[index_extended].logicals.size(); i++)
+				{
+					if (device.partitions[index_extended].logicals[i].type == TYPE_LOGICAL &&
+					    // Unless specifically told otherwise, the Linux kernel considers extended
+					    // boot records to be two sectors long, in order to "leave room for LILO".
+					    device.partitions[index_extended].logicals[i].sector_start
+					                                          - partition.sector_start < 2   )
+					{
+						partition.sector_start -= MEBIBYTE / partition.sector_size;
+					}
+				}
+			}
+		}
+	}
+
+	// Align end sector.
+	diff = (partition.sector_end + 1) % (MEBIBYTE / partition.sector_size);
+	if (diff)
+		partition.sector_end -= diff;
+
+	// If this is a logical partition not at end of drive then check to see if space
+	// is required for a following logical partition Extended Boot Record.
+	if (partition.type == TYPE_LOGICAL)
+	{
+		// If there is a following logical partition that starts less than 2 sectors
+		// from the end of this partition, then reserve at least a mebibyte for the EBR.
+		int index_extended = find_extended_partition(device.partitions);
+		if (index_extended >= 0)
+		{
+			for (unsigned int i = 0; i < device.partitions[index_extended].logicals.size(); i++)
+			{
+				if (device.partitions[index_extended].logicals[i].type         == TYPE_LOGICAL         &&
+				    device.partitions[index_extended].logicals[i].sector_start  > partition.sector_end &&
+				    // Unless specifically told otherwise, the Linux kernel considers extended
+				    // boot records to be two sectors long, in order to "leave room for LILO".
+				    device.partitions[index_extended].logicals[i].sector_start
+				                                                            - partition.sector_end < 2   )
+				{
+					partition.sector_end -= MEBIBYTE / partition.sector_size;
+				}
+			}
+		}
+
+		// If the logical partition end is beyond the end of the extended partition
+		// then reduce logical partition end by a mebibyte to address the overlap.
+		if (index_extended        != -1                                           &&
+		    partition.sector_end   > device.partitions[index_extended].sector_end   )
+		{
+			partition.sector_end -= MEBIBYTE / partition.sector_size;
+		}
+	}
+
+	// If this is a primary or an extended partition and the partition overlaps
+	// the start of the next primary or extended partition then subtract a
+	// mebibyte from the end of the partition to address the overlap.
+	if (partition.type == TYPE_PRIMARY || partition.type == TYPE_EXTENDED)
+	{
+		for (unsigned int i = 0; i < device.partitions.size(); i++)
+		{
+			if ((device.partitions[i].type == TYPE_PRIMARY ||
+			     device.partitions[i].type == TYPE_EXTENDED  )                      &&
+			     // For a change to an existing partition, (e.g., move or resize)
+			     // skip comparing to original partition and only compare to
+			     // other existing partitions.
+			    partition.status           == STAT_REAL                             &&
+			    partition.partition_number != device.partitions[i].partition_number &&
+			    device.partitions[i].sector_start  > partition.sector_start         &&
+			    device.partitions[i].sector_start <= partition.sector_end             )
+			{
+				partition.sector_end -= MEBIBYTE / partition.sector_size;
+			}
+		}
+	}
+
+	// If this is an extended partition then check to see if the end of the
+	// extended partition encompasses the end of the last logical partition.
+	if (partition.type == TYPE_EXTENDED)
+	{
+		// If there is logical partition that has an end sector beyond the
+		// end of the extended partition, then set the extended partition
+		// end sector to be the same as the end of the logical partition.
+		for (unsigned int i = 0; i < partition.logicals.size(); i++)
+		{
+			if (partition.logicals[i].type       == TYPE_LOGICAL         &&
+			    partition.logicals[i].sector_end  > partition.sector_end   )
+			{
+				partition.sector_end = partition.logicals[i].sector_end;
+			}
+		}
+	}
+
+	// If this is a GPT partition table and the partition ends less than 34 sectors
+	// from the end of the device, then reserve at least a mebibyte for the backup
+	// partition table.
+	if (device.disktype == "gpt" && device.length - partition.sector_end < 34)
+		partition.sector_end -= MEBIBYTE / partition.sector_size;
+}
+
+
 void Dialog_Base_Partition::Set_Confirm_Button( CONFIRMBUTTON button_type ) 
 { 
 	switch( button_type )
